Chuangjie Xu & Martin Escardo 2015

One of Brouwer's continuity principles is the following

    All functions ₂ℕ → ℕ are uniformly continuous

whose logical formulation is

    ∀(f : ₂ℕ → ℕ). ∃(m : ℕ). ∀(α,β : ₂ℕ). α =[m] β → f α = f β

where α =[m] β expresses that the sequences α and β agree up to the
first m positions.

\begin{code}

{-# OPTIONS --without-K #-}

module Continuity.EquivalentFormulationsOfUCWithoutTruncations where

open import Preliminaries.SetsAndFunctions
open import Preliminaries.NaturalNumber hiding (_+_)
open import Preliminaries.Boolean
open import Preliminaries.Sequence
open import Continuity.DecidabilityOfUC
open import Continuity.MainLemmas

\end{code}

If function extensionality is available, then for any type family A :
ℕ → U such that

  (1) A(n) is a proposition for all n,

  (2) if A(n) then A(m) is decidable for all i < n,

the truncation ∥ Σ(n:ℕ).A(n) ∥ exists by taking

    ∥ Σ(n:ℕ).A(n) ∥ = Σ-min(n:ℕ).A(n)

and, moreover, we have

    ∥ Σ(n:ℕ).A(n) ∥ → Σ(n:ℕ).A(n).

Notice that (funext) is only needed to prove that ∥ Σ(n:ℕ).A(n) ∥ is a
proposition.

\begin{code}

∥Σ_∥ : (ℕ → Set) → Set
∥Σ A ∥ = Σ-min A

∥Σ-∥-hprop : Funext →
             (A : ℕ → Set) →
             (∀ n → hprop (A n))
           → hprop ∥Σ A ∥
∥Σ-∥-hprop funext A hA (n , a , r) (n' , a' , r') = goal
 where
  claim₀ : n ≡ n'
  claim₀ = Lemma[m≤n∧n≤m→m=n] (r n' a') (r' n a)
  w : (A n') × (∀ m → A m → n' ≤ m)
  w = transport _ claim₀ (a , r)
  claim₁ : pr₁ w ≡ a'
  claim₁ = hA n' (pr₁ w) a'
  claim₂ : ∀(m : ℕ)(am : A m) → pr₂ w m am ≡ r' m am
  claim₂ m am = Lemma[≤-hprop] (pr₂ w m am) (r' m am)
  claim₃ : pr₂ w ≡ r'
  claim₃ = funext (λ m → funext (claim₂ m))
          --------      --------
  claim₄ : w ≡ (a' , r')
  claim₄ = pairˣ⁼ claim₁ claim₃
  goal : (n , a , r) ≡ (n' , a' , r')
  goal = pair⁼ claim₀ claim₄

ΣA→∥ΣA∥ : {A : ℕ → Set} →
          (∀ n → A n → ∀ m → m ≤ n → decidable (A m))
        → Σ A → ∥Σ A ∥
ΣA→∥ΣA∥ dA (n , a) = Lemma[Σ-min] n a (dA n a)

∥ΣA∥→ΣA : {A : ℕ → Set}
        → ∥Σ A ∥ → Σ A
∥ΣA∥→ΣA (n , a , _) = (n , a)

∥Σ-∥-elim : {A : ℕ → Set} {P : Set}
          → hprop P → (Σ A → P) → ∥Σ A ∥ → P
∥Σ-∥-elim _ f (n , a , _) = f (n , a)

\end{code}

Since Uc(f) : ℕ → U, defined by

  Uc(f,n) = (α β : ₂ℕ) → α ≡[ n ] β → f α ≡ f β

satisfies (1) and (2), the truncation of Uc(f) exists and hence we
have two formulations of the uniform continuity principle:

\begin{code}

UC : Set
UC = (f : ₂ℕ → ℕ) → ∥Σ (\(n : ℕ) → ∀(α β : ₂ℕ) → α ≡[ n ] β → f α ≡ f β) ∥

CH-UC : Set
CH-UC = (f : ₂ℕ → ℕ) → Σ \(n : ℕ) → ∀(α β : ₂ℕ) → α ≡[ n ] β → f α ≡ f β

\end{code}

Moreover, the above types are logically equivalent.

\begin{code}

Theorem[CH-UC→UC] : CH-UC → UC
Theorem[CH-UC→UC] chuc = λ f → ΣA→∥ΣA∥ (Uc-≤-decidable f) (chuc f) 

Theorem[UC→CH-UC] : UC → CH-UC
Theorem[UC→CH-UC] uc f = ∥ΣA∥→ΣA (uc f)

\end{code}
