Chuangjie Xu, November 2014

We prove the equivalence of the following formulations of sheaf condition:

(1) ∀(ρ : ₂ → ₂ℕ → X) → (∀ i → ρ i ∈ P) → Aml₂ ρ ∈ P

(2) ∀(n : ℕ) (ρ : ₂Fin n → ₂ℕ → X) → (∀ s → ρ s ∈ P) → Aml ρ ∈ P

(3) ∀(p : ₂ℕ → X) (n : ℕ) → (∀(s : ₂Fin n) → p ∘ cons s ∈ P) → p ∈ P

where Aml₂ ρ is the amalgamation of ρ₀ and ρ₁, and Aml ρ is the amalgamation of ρ s
as defined below.

All the above equivalences require function extensionality.

\begin{code}

module UsingFunext.EquivalenceSheafCondition where

open import Preliminaries.SetsAndFunctions hiding (_+_)
open import Preliminaries.NaturalNumber
open import Preliminaries.Boolean
open import Preliminaries.Sequence
open import UsingFunext.Funext

\end{code}

Preliminaries

\begin{code}


Aml₂ : {X : Set} → (₂ → ₂ℕ → X) → ₂ℕ → X
Aml₂ ρ α = ρ (α 0) (α ∘ succ)

Aml : {X : Set} {n : ℕ} → (₂Fin n → ₂ℕ → X) → ₂ℕ → X
Aml {X} {n} ρ α = ρ (take n α) (drop n α)


infixl 10 _‡_

_‡_ : {n : ℕ} → ₂Fin n → ₂ → ₂Fin (succ n)
⟨⟩ ‡ b      = b ∷ ⟨⟩
(h ∷ s) ‡ b = h ∷ (s ‡ b)

Lemma[‡] : {n : ℕ} (s : ₂Fin n) {α : ₂ℕ}
         → cons (s ‡ α 0) (α ∘ succ) ≣ cons s α
Lemma[‡] ⟨⟩       0       = refl
Lemma[‡] ⟨⟩      (succ i) = refl
Lemma[‡] (x ∷ s)  0       = refl
Lemma[‡] (x ∷ s) (succ i) = Lemma[‡] s i


Lemma[drop] : (n : ℕ) {α : ₂ℕ}
            → α n ≡ drop n α 0
Lemma[drop]  0       = refl
Lemma[drop] (succ n) = Lemma[drop] n

Lemma[‡]₁ : (n : ℕ) {α : ₂ℕ}
          → (take n α ‡ α n) ≡ take (succ n) α
Lemma[‡]₁  0       = refl
Lemma[‡]₁ (succ n) = ₂Fin-≡ refl (Lemma[‡]₁ n)

Lemma[‡]₂ : (n : ℕ) {α : ₂ℕ}
          → (take n α ‡ drop n α 0) ≡ take (succ n) α
Lemma[‡]₂ n {α} = transport (λ x → (take n α ‡ x) ≡ take (succ n) α)
                            (Lemma[drop] n) (Lemma[‡]₁ n)

Lemma[drop-succ] : (n : ℕ) {α : ₂ℕ}
                 → (drop n α) ∘ succ ≡ drop (succ n) α
Lemma[drop-succ]  0       = refl
Lemma[drop-succ] (succ n) = Lemma[drop-succ] n

ap² : {X Y Z : Set} {x₀ x₁ : X} {y₀ y₁ : Y}
    → (f : X → Y → Z) → x₀ ≡ x₁ → y₀ ≡ y₁ → f x₀ y₀ ≡ f x₁ y₁
ap² f refl refl = refl


Lemma[take-cons] : {n : ℕ} (s : ₂Fin n) {α : ₂ℕ}
                 → s ≡ take n (cons s α)
Lemma[take-cons] ⟨⟩      = refl
Lemma[take-cons] (x ∷ s) = ₂Fin-≡ refl (Lemma[take-cons] s)

Lemma[drop-cons] : {n : ℕ} (s : ₂Fin n) {α : ₂ℕ}
                 → α ≡ drop n (cons s α)
Lemma[drop-cons] ⟨⟩ = refl
Lemma[drop-cons] (_ ∷ s) = Lemma[drop-cons] s


\end{code}

The equivalence of

  ∀(ρ : ₂ → ₂ℕ → X) → (∀ i → ρ i ∈ P) → Aml₂ ρ ∈ P

and

  ∀(p : ₂ℕ → X) (n : ℕ) → (∀(s : ₂Fin n) → p ∘ cons s ∈ P) → p ∈ P

\begin{code}


Claim₀ : (X : Set) (P : (₂ℕ → X) → Set) →
         (∀(p : ₂ℕ → X) (n : ℕ) → (∀(s : ₂Fin n) → p ∘ cons s ∈ P) → p ∈ P) →
         ∀(ρ : ₂ → ₂ℕ → X) → (∀ i → ρ i ∈ P) → Aml₂ ρ ∈ P
Claim₀ X P c ρ ρf = c p 1 claim
 where
  p : ₂ℕ → X
  p = Aml₂ ρ
  claim : ∀(s : ₂Fin 1) → p ∘ cons s ∈ P
  claim (i ∷ ⟨⟩) = ρf i


Claim₁ : (X : Set) (P : (₂ℕ → X) → Set) →
         (∀(ρ : ₂ → ₂ℕ → X) → (∀ i → ρ i ∈ P) → Aml₂ ρ ∈ P) →
         ∀(p : ₂ℕ → X) → (n : ℕ) → (∀(s : ₂Fin n) → p ∘ cons s ∈ P) → p ∈ P
Claim₁ X P c p 0        pf = pf ⟨⟩
Claim₁ X P c p (succ n) pf = Claim₁ X P c p n pf'
 where
  pf' : ∀(s : ₂Fin n) → p ∘ cons s ∈ P
  pf' s = transport P claim claim₀
   where
    ρ : ₂ → ₂ℕ → X
    ρ i = p ∘ cons (s ‡ i)
    ρf : ∀ i → ρ i ∈ P
    ρf i = pf (s ‡ i)
    claim₀ : Aml₂ ρ ∈ P
    claim₀ = c ρ ρf
    claim₁ : ∀(α : ₂ℕ) → p (cons (s ‡ α 0) (α ∘ succ)) ≡ p (cons s α)
    claim₁ α = ap p (funext (Lemma[‡] s))
                    --------
    claim : Aml₂ ρ ≡ p ∘ cons s
    claim = funext claim₁
           --------


\end{code}

The equivalence of

  ∀(ρ : ₂ → ₂ℕ → X) → (∀ i → ρ i ∈ P) → Aml₂ ρ ∈ P

and

  ∀(n : ℕ) (ρ : ₂Fin n → ₂ℕ → X) → (∀ s → ρ s ∈ P) → Aml ρ ∈ P

\begin{code}


Claim₂ : (X : Set) (P : (₂ℕ → X) → Set) →
         (∀(n : ℕ) (ρ : ₂Fin n → ₂ℕ → X) → (∀ s → ρ s ∈ P) → Aml ρ ∈ P) →
         ∀(ρ : ₂ → ₂ℕ → X) → (∀ i → ρ i ∈ P) → Aml₂ ρ ∈ P
Claim₂ X P c ρ ρf = c 1 ρ' ρ'f
 where
  ρ' : ₂Fin 1 → ₂ℕ → X
  ρ' (i ∷ ⟨⟩) = ρ i
  ρ'f : ∀(s : ₂Fin 1) → ρ' s ∈ P
  ρ'f (i ∷ ⟨⟩) = ρf i


Claim₃ : (X : Set) (P : (₂ℕ → X) → Set) →
         (∀(ρ : ₂ → ₂ℕ → X) → (∀ i → ρ i ∈ P) → Aml₂ ρ ∈ P) →
         ∀(n : ℕ) (ρ : ₂Fin n → ₂ℕ → X) → (∀ s → ρ s ∈ P) → Aml ρ ∈ P
Claim₃ X P c  0       ρ ρf = ρf ⟨⟩
Claim₃ X P c (succ n) ρ ρf = transport P claim₁ IH
 where
  ρ₁ : ₂Fin n → ₂ → ₂ℕ → X
  ρ₁ s i = ρ (s ‡ i)
  ρ₂ : ₂Fin n → ₂ℕ → X
  ρ₂ s = Aml₂ (ρ₁ s)
  ρ₂f : ∀(s : ₂Fin n) → ρ₂ s ∈ P
  ρ₂f s = c (λ i → ρ (s ‡ i)) (λ i → ρf (s ‡ i))
  IH : Aml ρ₂ ∈ P
  IH = Claim₃ X P c n ρ₂ ρ₂f
  claim₀ : ∀(α : ₂ℕ) → ρ (take n α ‡ drop n α 0) ((drop n α) ∘ succ)
                     ≡ ρ (take (succ n) α) (drop (succ n) α)
  claim₀ α = ap² ρ (Lemma[‡]₂ n) (Lemma[drop-succ] n)
  claim₁ : Aml ρ₂ ≡ Aml ρ
  claim₁ = funext claim₀
          --------

\end{code}

The equivalence of

  ∀(p : ₂ℕ → X) (n : ℕ) → (∀(s : ₂Fin n) → p ∘ cons s ∈ P) → p ∈ P

and

  ∀(n : ℕ) (ρ : ₂Fin n → ₂ℕ → X) → (∀ s → ρ s ∈ P) → Aml ρ ∈ P

\begin{code}


Claim₄ : (X : Set) (P : (₂ℕ → X) → Set) →
         (∀(n : ℕ) (ρ : ₂Fin n → ₂ℕ → X) → (∀ s → ρ s ∈ P) → Aml ρ ∈ P) →
         ∀(p : ₂ℕ → X) (n : ℕ) → (∀(s : ₂Fin n) → p ∘ cons s ∈ P) → p ∈ P
Claim₄ X P c p n pf = transport P claim₂ claim₀
 where
  ρ : ₂Fin n → ₂ℕ → X
  ρ s = p ∘ cons s
  ρf : ∀(s : ₂Fin n) → ρ s ∈ P
  ρf s = pf s
  claim₀ : Aml ρ ∈ P
  claim₀ = c n ρ ρf
  claim₁ : ∀(α : ₂ℕ) → p (cons (take n α) (drop n α)) ≡ p α
  claim₁ α = ap p (funext (Lemma[cons-take-drop] n α))
                  --------
  claim₂ : Aml ρ ≡ p
  claim₂ = funext claim₁
          --------


Claim₅ : (X : Set) (P : (₂ℕ → X) → Set) →
         (∀(p : ₂ℕ → X) (n : ℕ) → (∀(s : ₂Fin n) → p ∘ cons s ∈ P) → p ∈ P) →
         ∀(n : ℕ) (ρ : ₂Fin n → ₂ℕ → X) → (∀ s → ρ s ∈ P) → Aml ρ ∈ P
Claim₅ X P c n ρ ρf = c p n pf
 where
  p : ₂ℕ → X
  p = Aml ρ
  pf : ∀(s : ₂Fin n) → p ∘ cons s ∈ P
  pf s = transport P claim₁ claim₂
   where
    claim₀ : ∀(α : ₂ℕ) → ρ s α ≡ ρ (take n (cons s α)) (drop n (cons s α))
    claim₀ α = ap² ρ (Lemma[take-cons] s) (Lemma[drop-cons] s)
    claim₁ : ρ s ≡ p ∘ cons s
    claim₁ = funext claim₀
            --------
    claim₂ : ρ s ∈ P
    claim₂ = ρf s


\end{code}

Verifying the sheaf condition (1) for discrete topology does not need funext.

But verifying (2) and (3) does require funext.

\begin{code}


LC : {X : Set} → (₂ℕ → X) → Set
LC p = Σ \(m : ℕ) → ∀(α β : ₂ℕ) → α ≡[ m ] β → p α ≡ p β

Claim₆ : {X : Set} (ρ : ₂ → ₂ℕ → X) → (∀ i → ρ i ∈ LC) → Aml₂ ρ ∈ LC
Claim₆ ρ ρf = m , prf
 where
  m₀ : ℕ
  m₀ = pr₁ (ρf ₀)
  m₁ : ℕ
  m₁ = pr₁ (ρf ₁)
  lemma : ∀ i → pr₁ (ρf i) ≤ max m₀ m₁
  lemma ₀ = max-spec₀ m₀ m₁
  lemma ₁ = max-spec₁ m₀ m₁
  m : ℕ
  m = succ (max m₀ m₁)
  prf : ∀(α β : ₂ℕ) → α ≡[ m ] β → Aml₂ ρ α ≡ Aml₂ ρ β
  prf α β em = claim₃ · claim₄
   where
    claim₀ : α 0 ≡ β 0
    claim₀ = Lemma[≡[]-zero] em
    claim₁ : α ∘ succ ≡[ max m₀ m₁ ] β ∘ succ
    claim₁ = Lemma[≡[]-succ] em
    claim₂ : α ∘ succ ≡[ pr₁ (ρf (β 0)) ] β ∘ succ
    claim₂ = Lemma[≡[]-≤] claim₁ (lemma (β 0))
    claim₃ : ρ (α 0) (α ∘ succ) ≡ ρ (β 0) (α ∘ succ)
    claim₃ = ap (λ i → ρ i (α ∘ succ)) claim₀
    claim₄ : ρ (β 0) (α ∘ succ) ≡ ρ (β 0) (β ∘ succ)
    claim₄ = pr₂ (ρf (β 0)) (α ∘ succ) (β ∘ succ) claim₂


\end{code}
