Martin Escardo & Chuangjie Xu, 15 Sep 2014

\begin{code}

module UsingNotNotFunext.Separatedness where

open import Preliminaries.SetsAndFunctions
open import UsingNotNotFunext.NotNot
open import UsingFunext.Funext

\end{code}

Definitions

\begin{code}

separated : Set → Set
separated X = ∀(x x' : X) → ¬¬ (x ≡ x') → x ≡ x'

×-separated : {X Y : Set} → separated X → separated Y → separated (X × Y)
×-separated sX sY (x , y) (x' , y') rh = pairˣ⁼ (sX x x' ph) (sY y y' qh)
 where
  ph : ¬¬ (x ≡ x')
  ph = functor pr₁⁼ rh
  qh : ¬¬ (y ≡ y')
  qh = functor pr₂ˣ⁼ rh

→-separated : {X Y : Set} → separated Y → separated (X → Y)
→-separated sY f f' rh = funext claim₃
                        --------
 where
  claim₀ : f ≡ f' → ∀ x → f x ≡ f' x
  claim₀ = fun-ap
  claim₁ : ¬¬ (∀ x → f x ≡ f' x)
  claim₁ = functor claim₀ rh
  claim₂ : ∀ x → ¬¬ (f x ≡ f' x)
  claim₂ = Lemma[[X→Y]⇒X→[Y]] claim₁
  claim₃ : ∀ x → f x ≡ f' x
  claim₃ x = sY (f x) (f' x) (claim₂ x)

\end{code}

Lemmas

\begin{code}

dep-functor : {X : Set}(Y : ¬¬ X → Set) → ((x : X) → Y(hide x)) → (xh : ¬¬ X) → ¬¬ (Y xh)
dep-functor {X} Y f xh = extension claim xh
 where
  claim : X → ¬¬ (Y xh)
  claim x g = g claim₁
   where
    claim₀ : hide x ≡ xh
    claim₀ = funext (λ h → ∅-elim (h x))
            --------
    claim₁ : Y xh
    claim₁ = transport Y claim₀ (f x)

postulate
 separated-is-hset : Funext → {X : Set} → separated X → hset X

\end{code}

Σ preserves separatedness

\begin{code}

Σ-separated : (X : Set) (Y : X → Set)
            → separated X → (∀ x → separated (Y x)) → separated (Σ Y)
Σ-separated X Y sX sY (x , y) (x' , y') hw = pair⁼ p q
 where
  hp : ¬¬ (x ≡ x')
  hp = functor pr₁⁼ hw
  p : x ≡ x'
  p = sX x x' (functor pr₁⁼ hw)
  Q : ¬¬ ((x , y) ≡ (x' , y')) → Set
  Q hw = transport Y (sX x x' (functor pr₁⁼ hw)) y ≡ y'
  claim : ∀(w : (x , y) ≡ (x' , y'))
        → pr₁⁼ w ≡ sX x x' (functor pr₁⁼ (hide w))
  claim w = separated-is-hset funext sX (pr₁⁼ w) (sX x x' (functor pr₁⁼ (hide w)))
                             --------
  f : (w : (x , y) ≡ (x' , y'))
    → transport Y (sX x x' (functor pr₁⁼ (hide w))) y ≡ y'
  f w = transport (λ e → transport Y e y ≡ y') (claim w) (pr₂⁼ w)
  hq : ¬¬ (transport Y p y ≡ y')
  hq = dep-functor Q f hw
  q : transport Y p y ≡ y'
  q = sY x' (transport Y p y) y' hq

\end{code}

Π preserves separatedness

\begin{code}

Π : {X : Set} → (X → Set) → Set
Π {X} Y = (x : X) → Y x

Π-separated : (X : Set) (Y : X → Set)
            → separated X → (∀ x → separated (Y x)) → separated (Π Y)
Π-separated X Y sX sY f g hw = funext claim₂
                              --------
 where
  claim₀ : ¬¬ (∀ x → f x ≡ g x)
  claim₀ = functor fun-ap hw
  claim₁ : ∀ x → ¬¬ (f x ≡ g x)
  claim₁ x = functor(λ h → h x) claim₀
  claim₂ : ∀ x → f x ≡ g x
  claim₂ x = sY x (f x) (g x) (claim₁ x)

\end{code}
